//
//  Copyright (C) 2011-2012 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


namespace Footnote.Widgets {
    /**
     * class representing a note
     **/
    public class Note : GtkClutter.Actor {
        public int BUTTON_WIDTH = 95;

        public Gtk.Label        title;
        public Gtk.TextView     content;
        public Gtk.Label        date;
        public Notebook         notebook;
        public Gtk.ToggleButton bookmarked;
        public Gtk.TreeIter     miller_iter; //iter used for miller treeview
        
        FootnoteApp         app;
        
        public int          id;
        public int          month;
        public int          year;
        
        public bool is_brand_new;
        public bool deleted;
        
        public Gtk.TreeIter bookmarked_iter; //may be null
        
        Gdk.Atom format;
        Gdk.Atom deformat;
        
        public Note.from_existing (FootnoteApp app, Notebook notebook, 
            string title, uint8[] content, int id, string date, bool bookmarked){
            this.app            = app;
            
            this.build (app);
            
            this.title.label       = title;
            this.notebook          = notebook;
            this.id                = id;
            this.date.label        = date;
            this.bookmarked.active = bookmarked;
            
            if (content != null) {
                Gtk.TextIter s;
                this.content.buffer.get_start_iter (out s);
                try {
                    this.content.buffer.deserialize (this.content.buffer, deformat, s, content);
                } catch (Error e) { warning (e.message); }
            }
            
            int m = date.index_of (".");
            int y = date.index_of (".", m+1);
            this.month = int.parse (date.slice (m+1, y));
            this.year  = int.parse (date.substring (y+1));
            
            if (app.displayed_years.find (this.year) == null) { //add not added years to the filter comboxbox
                app.displayed_years.append (this.year);
                app.years.append (this.year.to_string (), this.year.to_string ());
            }
        }
        
        public void tag (string name, bool add) {
            Gtk.TextIter s, e;
            this.content.buffer.get_selection_bounds (out s, out e);
            if (add)
                this.content.buffer.apply_tag_by_name (name, s, e);
            else
                this.content.buffer.remove_tag_by_name (name, s, e);
        }
        
        private void build (FootnoteApp app) {
            
            var box         = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
            var grid        = new Gtk.Grid ();
            this.title      = new LLabel ("");
            this.content    = new Gtk.TextView ();
            this.date       = new LLabel ("");
            var export      = new Gtk.Button.with_label (_("Export"));
            var edit        = new Gtk.Button.with_label (_("Edit"));
            var del         = new Gtk.Button.with_label (_("Delete"));
            this.bookmarked = new Gtk.ToggleButton ();
            var buttons     = new Gtk.Box (Gtk.Orientation.HORIZONTAL, 12);
            this.format     = this.content.buffer.register_serialize_tagset ("default");
            this.deformat     = this.content.buffer.register_deserialize_tagset ("default");
            
            this.get_widget ().get_style_context ().add_class ("content-view");
            
            this.title.selectable = true;
            this.title.modify_font (Pango.FontDescription.from_string ("bold 15"));
            
            this.date.modify_font (Pango.FontDescription.from_string ("10"));
            this.date.margin_top = this.date.margin_bottom = 6;
            this.date.selectable = true;
            
            this.content.modify_font (Pango.FontDescription.from_string ("12"));
            this.content.editable = false;
            this.content.cursor_visible = false;
            this.content.wrap_mode = Gtk.WrapMode.WORD_CHAR;
            this.content.valign = Gtk.Align.START;
            this.content.buffer.create_tag ("bold", "weight", Pango.Weight.BOLD);
            this.content.buffer.create_tag ("italic", "style", Pango.Style.ITALIC);
            this.content.buffer.create_tag ("strikethrough", "strikethrough", true);
            this.content.buffer.create_tag ("marked", "background", "#ffff00");
            
            this.bookmarked.halign = Gtk.Align.END;
            this.bookmarked.valign = Gtk.Align.CENTER;
            this.bookmarked.add (new Gtk.Image.from_icon_name ("bookmark-new", Gtk.IconSize.BUTTON));
            
            this.is_brand_new = true;

            export.width_request = BUTTON_WIDTH;
            edit.width_request = BUTTON_WIDTH;
            del.width_request = BUTTON_WIDTH;

            buttons.hexpand = true;
            buttons.pack_start (export, false);
            buttons.pack_start (new Gtk.Label(""));
            buttons.pack_start (edit, false);
            buttons.pack_start (del, false);
            buttons.margin_top = 12;
            
            grid.margin = 12;
            grid.attach (this.title,      0, 0, 1, 1);
            grid.attach (this.date,       0, 1, 1, 1);
            grid.attach (this.bookmarked, 1, 0, 1, 2);
            grid.attach (this.content,    0, 2, 2, 1);
            grid.attach (buttons,         0, 3, 2, 1);
            
            box.pack_start (grid, false);
            box.pack_start (new Gtk.Separator (Gtk.Orientation.HORIZONTAL), false);
            
            ((Gtk.Container)this.get_widget ()).add (box);
            /**
             * Events
             **/
            /*ask the app to edit the note*/
            edit.clicked.connect ( () => {
                app.edit_view.open_note (this);
            });
            /*DnD*/
            this.get_widget ().add_events (Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK);
            Gtk.TargetEntry e = {"footnote_note", 0, 0};
            Gtk.drag_source_set (this.get_widget (), Gdk.ModifierType.BUTTON1_MASK | 
                                       Gdk.ModifierType.BUTTON3_MASK, {e}, Gdk.DragAction.MOVE);
            this.get_widget ().drag_begin.connect ( (context) => {
                app.selected.note = this;
                app.selected.notebook = this.notebook; //just make sure...
            });
            this.get_widget ().drag_data_get.connect ( (ctx, sel, info, time) => {
                sel.set (sel.get_target (), 8, this.id.to_string ().data);
            });
            /*export clicked, show contractor dialog*/
            export.clicked.connect ( () => {
                var dialog = new Gtk.Dialog.with_buttons ("Export", app.mainwindow, Gtk.DialogFlags.MODAL,
                     Gtk.Stock.CANCEL, Gtk.ResponseType.REJECT, Gtk.Stock.OK, Gtk.ResponseType.ACCEPT);
                var contr = new Granite.Widgets.ContractorView (save_temp_file (), "text");
                contr.name_blacklist ({"Brasero"});
                contr.margin = 12;
                ((Gtk.Box)dialog.get_content_area ()).add (contr);
                
                dialog.show_all ();
                if (dialog.run () == Gtk.ResponseType.ACCEPT)
                    contr.run_selected ();
                dialog.destroy ();
            });
            /*delete note*/
            del.clicked.connect ( () => {
                
                this.deleted = true;
                nt.deleted_notes.append (this);
                
                var hbox = new Gtk.Box (Gtk.Orientation.HORIZONTAL, 12);
                var vbox = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
                hbox.hexpand = true;
                hbox.margin = 12;
                var ebox = new Gtk.EventBox();
                var css = new Gtk.CssProvider ();
                
                try {
                    css.load_from_data ("*{background-color: @bg_color;}", -1);
                } catch (Error e) { warning (e.message); }
                
                ebox.get_style_context ().add_provider (css, 40000);
                
                var note_label = "<b>" + this.title.get_text() + "</b>" + _(" was deleted");
                var note_name = new LLabel.markup (note_label);
                var dismiss_button = new Gtk.Button.with_label (_("Dismiss"));
                dismiss_button.set_halign (Gtk.Align.END);
                dismiss_button.width_request = BUTTON_WIDTH;
                var undo_button = new Gtk.Button.with_label (_("Undo"));
                undo_button.set_halign (Gtk.Align.END);
                undo_button.width_request = BUTTON_WIDTH;
                
                hbox.pack_start (note_name, false);
                hbox.pack_start (new Gtk.Label(""));
                hbox.pack_start (dismiss_button, false);
                hbox.pack_start (undo_button, false);
                       
                vbox.pack_start (hbox);
                vbox.pack_start (new Gtk.Separator (Gtk.Orientation.HORIZONTAL), false);
                
                ebox.add (vbox);
                ebox.show_all();
                
                (get_widget () as Gtk.Container).remove (box);
                (get_widget () as Gtk.Container).add  (ebox);
                (get_widget () as Gtk.Container).show_all ();
            
                dismiss_button.clicked.connect ( () => {
                    app.selected.note = this; //just make sure..
                    (get_widget () as Gtk.Container).remove (ebox);
                    nt.deleted_notes.remove (this);
                    action_delete_note ();
                });
                
                undo_button.clicked.connect ( () => {
                    this.deleted = false;
                
                    (get_widget () as Gtk.Container).remove (ebox);
                    (get_widget () as Gtk.Container).add  (box);
                });

            });
            this.bookmarked.toggled.connect ( () => {
                this.bookmarked.foreach ( (w) => this.bookmarked.remove (w) );
                if (this.bookmarked.active) {
                    this.bookmarked.add (new Gtk.Image.from_icon_name ("help-about", Gtk.IconSize.BUTTON));
                    try {
                        this.bookmarked_iter = app.sidebar.addItem (app.bookmarks_iter, this, null, 
                            Gtk.IconTheme.get_default ().load_icon ("text", 16, 0), this.title.label, null);
                    } catch (Error e) { warning (e.message); }
                } else {
                    this.bookmarked.add (new Gtk.Image.from_icon_name ("bookmark-new", Gtk.IconSize.BUTTON));
                    app.sidebar.removeItem (this.bookmarked_iter);
                    this.bookmarked_iter = {};
                }
                this.bookmarked.show_all ();
                this.save ();
            });
            /*update scroll height*/
            this.notify["height"].connect ( () => this.notebook.view.calculate_height ());
            
            this.get_widget ().show_all ();
            
            this.scale_gravity = Clutter.Gravity.NORTH;
        }
        
        /*override some functions for animations*/
        public new void hide () {
            var a = this.animate (Clutter.AnimationMode.EASE_OUT_QUAD, 400, height:0.0f, opacity:0);
            a.completed.connect ( () => base.hide () );
        }
        public new void show () {
            base.show ();
            this.animate (Clutter.AnimationMode.EASE_OUT_QUAD, 400, height:-1.0f, opacity:255);
        }
        
        /*return string including markup for miller view tree*/
        public string get_miller_label () {
            var c = this.content.buffer.text;
            var i = c.index_of ("\n");
            if (i != -1)
                c = c.substring (0, i);
            return this.title.label+"\n<span color='#ccc' font='10'>"+this.date.label+"</span>\n<span font='10'>"+c+"</span>";
        }
        
        public string save_temp_file () {
            var path = GLib.Environment.get_tmp_dir () + "/" + this.title.label + ".txt";
            debug ("Saving temp file to %s", path);
            try {
                GLib.File file = GLib.File.new_for_path (path);
                uint8[] data = content.buffer.text.data;
                string s;
                file.replace_contents (data, null, false, 0, out s);
            } catch (GLib.Error e) { warning (e.message); }
            return path;
        }
        
        public Note (FootnoteApp app, Notebook notebook) {
            this.build (app);
            
            this.notebook       = notebook;
            this.app            = app;
            this.title.label    = _("Untitled");
            
            /*setup date*/
            var dt              = new DateTime.now_local ();
            var min             = (dt.get_minute () < 10)?"0"+dt.get_minute ().to_string ():dt.get_minute ().to_string ();
            var hour            = (dt.get_hour () < 10)?"0"+dt.get_hour ().to_string ():dt.get_hour ().to_string ();
            string[] weekdays   = {_("Monday"),_("Tuesday"),_("Wednesday"),_("Thursday"),
                _("Friday"),_("Saturday"),_("Sunday")};
            this.date.label     = hour+":"+min+" "+weekdays[dt.get_day_of_week ()-1]+", "+
                dt.get_day_of_month ().to_string ()+"."+dt.get_month ().to_string ()+"."+
                dt.get_year ().to_string ();
            
            this.month = dt.get_month ();
            this.year  = dt.get_year ();
            
            try {
                var q = new SQLHeavy.Query (app.db, "INSERT INTO 'notes' ("+
                    "title, content, notebook, date, bookmarked) VALUES  ("+
                    ":title, :content, :notebook, :date, :bookmarked);");
                q.set_string (":title", _("Untitled"));
                q.set_string (":content", "");
                q.set_int (":notebook", this.notebook.id);
                q.set_string (":date", this.date.label);
                q.set_int (":bookmarked", (int)this.bookmarked.active);
                q.execute ();
                this.id = (int)app.db.last_insert_id;
            } catch (SQLHeavy.Error e) { warning ("Could not create note, %s", e.message); }
            this.notebook = notebook;
            
            this.app.selected.note = this;
        }
        
        public void save () {
            try {
                Gtk.TextIter s, e;
                this.content.buffer.get_bounds (out s, out e);
                uint8 [] data = this.content.buffer.serialize (this.content.buffer, format, s, e);
                
                var q = app.db.prepare ("UPDATE 'notes' SET title=:title, content=:content, bookmarked=:bookmarked WHERE rowid=:id");
                q.set_string (":title", this.title.label);
                q.set_blob (":content", data);
                q.set_int (":bookmarked", (int)this.bookmarked.active);
                q.set_int (":id", this.id);
                q.execute ();
                if (this.is_brand_new == true)
                	this.is_brand_new = false;
            } catch (SQLHeavy.Error e) { warning ("Problem saving note, %s", e.message); }
            if (this.bookmarked.active)
                app.sidebar.setName (app.sidebar.convertToFilter (this.bookmarked_iter), this.title.label);
            
            this.notebook.view.miller_store.set (this.miller_iter, 0, this.get_miller_label ());
        }
        
        public void del_from_db () {
            if (this.bookmarked.active)
                app.sidebar.removeItem (this.bookmarked_iter);
            try {
                var q = new SQLHeavy.Query (app.db, "DELETE FROM 'notes' WHERE rowid = :id;");
                q.set_int (":id", this.id);
                q.execute ();
            } catch (SQLHeavy.Error e) { warning ("Could not delete note, %s", e.message); }
        }
        
    }
}


