//
//  Copyright (C) 2011-2012 Tom Beckmann
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

namespace Footnote.Widgets {
    /**
     * Class represeting a Notebook with its notes
     **/
    public class Notebook : GLib.Object {
        public NoteView     view;
        public int          id;
        public string       title;
        public Gtk.TreeIter iter;
        public int          count;
        private Gdk.Pixbuf  line;
        public uint32       color;
        public List<Note>   notes;
        private FootnoteApp app;
        
        public Notebook.from_existing (FootnoteApp app, string title, uint32 color, int id) {
            this.app = app;
            
            try {
                this.notes  = new List<Note> ();
                this.id     = id;
                this.title  = title;
                this.color  = color;
                
                this.build (app);
                
                /*get the notes together*/
                var q = new SQLHeavy.Query (app.db, "SELECT title,content,rowid,date,bookmarked FROM 'notes' WHERE notebook = :id ORDER BY rowid ASC;");
                q.set_int (":id", this.id);
                for (var res = q.execute ();!res.finished;res.next ()){
                    Note note = new Note.from_existing (app, this, res.fetch_string (0), res.fetch_blob (1), 
                        res.fetch_int (2), res.fetch_string (3), (bool)res.fetch_int (4));
                    this.notes.append (note);
                    this.view.add_note (note);
                }
            } catch (SQLHeavy.Error e) { warning ("Could not query notes, %s", e.message); }
            
            this.app.notebooks.append (this);
        }
        
        public Notebook (FootnoteApp app, string title, uint8 r, uint8 g, uint8 b){
            if (title == "") //dont want unnamed notebook
                title = _("Untitled");
            this.app = app;
            this.notes = new List<Note> ();
            this.title = title;
            this.color = this.get_color (r, g, b);
            try {
                var q = app.db.prepare ("INSERT INTO 'notebooks' (name, color) VALUES (:name, :color);");
                q.set_string (":name", this.title);
                q.set_int64 (":color", this.color);
                q.execute ();
                this.id = (int)app.db.last_insert_id;
            } catch (SQLHeavy.Error e) { warning ("Could not create notebook, %s", e.message); }
            
            this.build (app);
            
            // Dont try to hide when it is not set yet!
            if (app.selected.notebook != null) {
                app.selected.notebook.view.hide ();
            }

            app.selected.notebook = this;
            app.note_view.child = app.selected.notebook.view;
            app.selected.notebook.view.update_view ();
            
            var n = new Note (app, this);
            this.add_note (n);
            app.edit_view.open_note (n);
            
            this.view.show_all ();
            if (app.view_mode.selected == 0)
                this.view.miller_paned.hide ();
            else
                this.view.clutter.hide ();
            
            this.view.update_view ();
        }
        
        private void edit_cat (FootnoteApp app, string title, Granite.Widgets.PopOver dlg, uint8 r, uint8 g, uint8 b){
            if (title != ""){
                this.title = title;
                this.color = get_color (r,g,b);
                app.remove_button.set_sensitive(true);
                app.main_actions.get_action ("NewNote").set_sensitive (true);
                dlg.destroy ();
                try {
                    var q = new SQLHeavy.Query (app.db, "UPDATE 'notebooks' SET name=:name, color=:color WHERE id=:id");
                    q.set_string (":name", this.title);
                    q.set_int64 (":color", (int64)this.color);
                    q.set_int(":id", this.id);
                    q.execute ();
                } catch (SQLHeavy.Error e) { warning (e.message); }
                
                line.fill (this.color);
                
                app.sidebar.setName (app.sidebar.convertToFilter (this.iter), this.title);
                app.sidebar.setPixbuf (app.sidebar.convertToFilter (this.iter), this.line);
            }
        }
        
        private uint32 get_color (uint8 r, uint8 g, uint8 b){
            return (uint32)((r<<24)+(g<<16)+(b<<8)+0x00000000);
        }
        
        private double[] from_color (uint32 color) {
            uint32 r = color>>24;
            uint32 g = color>>16;
            uint32 bl = color>>8;
            return {(double)((uint8)r)/255.0,(double)((uint8)g)/255.0,(double)((uint8)bl)/255.0};
        }
        
        private void build (FootnoteApp app){
            var col = from_color (this.color);
            this.line = Footnote.Widgets.ColorRow.get_color_dot (col[0], col[1], col[2], 15, 15);
            
            this.iter = app.sidebar.addItem (app.notebooks_iter, this, this.view, this.line, this.title, null);
            this.view = new NoteView (app);
            
            app.note_view.pack_start (this.view);
        }
        
        public void edit (FootnoteApp app){
            var dlg  = new Granite.Widgets.PopOver ();
            var name = new Gtk.Entry ();
            var edit = new Gtk.Button.from_stock (Gtk.Stock.EDIT);
            var b    = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
            var colb = new Gtk.ColorButton ();
            
            name.set_placeholder_text (_("Notebook name"));
            name.set_text (this.title);
            
            double[] rgb = this.from_color (this.color);
            var curColor = Gdk.RGBA () {red=rgb[0],green=rgb[1],blue=rgb[2]};
            colb.set_rgba (curColor);
            b.pack_start (new Gtk.Label (_("Name")));
            b.pack_start (name);
            b.pack_start ( new Gtk.Label (_("Color")));
            b.pack_start (colb);
            b.pack_start (edit);
            
            name.key_release_event.connect ( (event)=>{
                if (name.text != ""){
                    edit.set_sensitive (true);
                    if  (event.keyval == 65293)
                        edit.clicked ();
                }else{
                    edit.set_sensitive (false);
                }
                return false;
            });
            edit.clicked.connect ( ()=> {
                this.edit_cat (app, name.text, dlg, (uint8)colb.color.red, (uint8)colb.color.green, (uint8)colb.color.blue);
            });
            
            ((Gtk.Box)dlg.get_content_area ()).add (b);
            dlg.show_all ();
            dlg.move_to_widget (app.edit_button);
            dlg.present ();
            dlg.run ();
            dlg.destroy ();
        }
        
        public void del (FootnoteApp app){
            var dlg = new Granite.Widgets.PopOver ();
            var b   = new Gtk.Box (Gtk.Orientation.VERTICAL, 0);
            var yes = new Gtk.Button.from_stock (Gtk.Stock.DELETE);
            
            b.pack_start (new Gtk.Label (_("Delete topic with all its notes?")));
            b.pack_start (yes);
            ((Gtk.Box)dlg.get_content_area ()).add (b);
            
            yes.get_style_context ().add_class ("noundo");
            yes.clicked.connect ( ()=>{
                try{
                    SQLHeavy.Query q = new SQLHeavy.Query (app.db, "DELETE FROM 'notebooks' WHERE id = :id;");
                    q.set_int (":id", this.id);
                    q.execute ();
                    var r = new SQLHeavy.Query (app.db, "DELETE FROM 'notes' WHERE notebook = :id;");
                    r.set_int (":id", this.id);
                    r.execute ();
                } catch (SQLHeavy.Error e) { warning (e.message); }
                
                app.notebooks.delete_link (app.notebooks.find (this));
                app.sidebar.removeItem (this.iter);
                
                this.notes.foreach ( (n) => {
                    if (n.bookmarked.active)
                        app.sidebar.removeItem (n.bookmarked_iter);
                });
                
                if (app.notebooks.length () != 0){
                    app.selected.notebook = app.notebooks.nth_data (0);
                    app.main_actions.get_action ("NewNote").set_sensitive (true);
                } else {
                    app.selected.notebook = null;
                    app.main_actions.get_action ("NewNote").set_sensitive (false);
                    app.remove_button.set_sensitive (false);
                }
                
                foreach (Note n in notes)
                    n.del_from_db ();
                
                this.view.hide ();
                
                dlg.destroy ();
            });
            dlg.show_all ();
            dlg.move_to_widget (app.remove_button);
            dlg.present ();
            dlg.run ();
            dlg.destroy ();
        }
        
        public void add_note (Note n) {
            if (notes.length () > 0)
                this.notes.insert (n, 1);
            else
                this.notes.insert (n, 0);
            this.view.add_note (n);
        }
        
        public void delete_note (Note n) {
            n.del_from_db ();
            
            this.view.remove_note (n);
            this.view.miller_store.remove (n.miller_iter);
            this.notes.delete_link (this.notes.find (n));
            this.view.update_view ();
        }
    }
}
